using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace AlienAttack
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class AlienAttackGame : Microsoft.Xna.Framework.Game
	{
		private GraphicsDeviceManager graphics;
		private SpriteBatch spriteBatch;
		public static AudioManager AudioManager;

		private Texture2D bgScreen;
		private SpriteFont arial;
		private Player player;
		private List<PlayerShot> playerShots;
		private EnemyGroup enemyGroup;
		private Explosion playerExplosion;

		private int score;
		private double lastTime;
		private float PlayerVelocity = ScreenWidth / 200.0f;

#if ZUNE
		public static int ScreenWidth = 240;
		public static int ScreenHeight = 320;
#else
		public static int ScreenWidth = 1024;
		public static int ScreenHeight = 768;
#endif

		public AlienAttackGame()
		{
			graphics = new GraphicsDeviceManager(this);

			// set our screen size based on the device
			graphics.PreferredBackBufferWidth = ScreenWidth;
			graphics.PreferredBackBufferHeight = ScreenHeight;

			Content.RootDirectory = "Content";
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			// create the audio helper
			AudioManager = new AudioManager(this.Content);

			AudioManager.PlayCue(AudioManager.Cue.Theme);

			player = new Player(Content);

			playerShots = new List<PlayerShot>();

			enemyGroup = new EnemyGroup(Content);

			base.Initialize();
		}

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		protected override void LoadContent()
		{
			// Create a new SpriteBatch, which can be used to draw textures.
			spriteBatch = new SpriteBatch(GraphicsDevice);

			bgScreen = Content.Load<Texture2D>("gfx\\bgScreen");
			arial = Content.Load<SpriteFont>("arial");
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// all content.
		/// </summary>
		protected override void UnloadContent()
		{
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{
			// update the user input
			InputManager.Update();

			// Allows the game to exit
			if(InputManager.ControlState.Quit)
				this.Exit();

			MovePlayer(gameTime);
			UpdatePlayerShots(gameTime);

			enemyGroup.Update(gameTime);

			HandleCollisions(gameTime);

			base.Update(gameTime);
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			// open the spritebatch, draw the screen, close it up
			this.spriteBatch.Begin();

			// draw the background
			spriteBatch.Draw(bgScreen, Vector2.Zero, Color.White);

			// draw the player
			if(player != null)
				player.Draw(gameTime, spriteBatch);

			// draw the enemy board
			enemyGroup.Draw(gameTime, spriteBatch);

			// draw the player shots
			foreach(PlayerShot playerShot in playerShots)
				playerShot.Draw(gameTime, spriteBatch);

			// draw the explosion
			if(playerExplosion != null)
				playerExplosion.Draw(gameTime, spriteBatch);

#if ZUNE
			// draw the score
			spriteBatch.DrawString(arial, "Score", new Vector2(0, 0), Color.White);
			spriteBatch.DrawString(arial, score.ToString(), new Vector2(0, 20), Color.White);
#else
			// draw the score
			spriteBatch.DrawString(arial, "Score", new Vector2(50, 50), Color.White);
			spriteBatch.DrawString(arial, score.ToString(), new Vector2(50, 80), Color.White);
#endif

			// draw the proper text, if required
			if(enemyGroup.AllDestroyed())
			{
				Vector2 size = arial.MeasureString("You win!");
				spriteBatch.DrawString(arial, "You win!", new Vector2((ScreenWidth - size.X) / 2, (ScreenHeight - size.Y) / 2), Color.Green);
			}

			this.spriteBatch.End();
		}

		private void HandleCollisions(GameTime gameTime)
		{
			// see if a player shot hit an enemy
			for(int i = 0; i < playerShots.Count; i++)
			{
				PlayerShot playerShot = playerShots[i];
				// check the shot and see if it it collided with an enemy
				if(playerShot != null && enemyGroup.HandlePlayerShotCollision(playerShots[i]))
				{
					// remove the shot, add the score
					playerShots.RemoveAt(i);
					score += 100;
					AudioManager.PlayCue(AudioManager.Cue.Explosion);
				}
			}

			// see if an enemy shot hit the player
			if(player != null && enemyGroup.HandleEnemyShotCollision(player))
			{
				// blow up the player
				playerExplosion = new Explosion(Content);
				playerExplosion.Position = player.Position;
				player = null;
				AudioManager.PlayCue(AudioManager.Cue.Explosion);
			}

			// if the player explosion animation is running, update it
			if(playerExplosion != null && playerExplosion.Update(gameTime))
			{
				// remove it
				playerExplosion = null;

				enemyGroup.Reset();
				playerShots.Clear();
				player = new Player(Content);
			}
		}

		private void UpdatePlayerShots(GameTime gameTime)
		{
			// if we are allowed to fire, add a shot to the list
			if(player != null && InputManager.ControlState.Fire && gameTime.TotalGameTime.TotalMilliseconds - lastTime > 500)
			{
				// create a new shot over the ship
				PlayerShot ps = new PlayerShot(Content);
				ps.Position.X = (player.Position.X + player.Width/2) - ps.Width/2;
				ps.Position.Y = player.Position.Y - ps.Height;
				playerShots.Add(ps);
				lastTime = gameTime.TotalGameTime.TotalMilliseconds;
				AudioManager.PlayCue(AudioManager.Cue.PlayerShot);
			}

			// enumerate the player shots on the screen
			for(int i = 0; i < playerShots.Count; i++)
			{
				PlayerShot playerShot = playerShots[i];

				playerShot.Update(gameTime);

				// if it's off the top of the screen, remove it from the list
				if(playerShot.Position.Y + playerShot.Height < 0)
					playerShots.RemoveAt(i);
			}
		}

		private void MovePlayer(GameTime gameTime)
		{
			if(player != null)
			{
				// move left
				if(InputManager.ControlState.Left && player.Position.X > 0)
					player.Position.X -= PlayerVelocity;

				// move right
				if(InputManager.ControlState.Right && player.Position.X + player.Width < ScreenWidth)
					player.Position.X += PlayerVelocity;

				player.Update(gameTime);
			}
		}
	}
}
